/*
	Arabian Map Export To Lua
	Script which used by Arabian Bandits (AB) Game Developer to Export, with correct format, from AB Adobe Photoshop File to <map-file>.lua
	The lua file will be read by MapReader.lua in AB's Code
*/

var docRef = app.activeDocument; 	// Reference to active document
var layers = docRef.layers; 		// Reference to active layers
var output = ""; 					// The output string that will be written to output file
var orderLayer 	= "sorted"; 		// Order layer, there are 5 type of layers (Top, Upper, Sorted, Center, Bottom)
var section 	= "nil"; 			// Section number, can be used in output function to get current section number, formatted in quote (e.g: section one will be written as "1")
var appearOrder	= ""; 				// appear and/or disappear configuration 
var dayPreference = "dayPreference = nil, "; 			// noon or night preference 

var layerName; 
var tx;
var ty; 
var res_width;
var res_height; 
var pos_x;
var pos_y;

/*
	Output Format:
	1. HEADER STRING (Variable declaration, and Map Atrributes)
	2. START POSITION
	3. SECTIONS' DATA (descending order, as many as available sections)
		a. Section's attributes
		b. Portals
		c. Enemy Triggers, etc (if any)
		d. Objects, Buildings
		e. Background 
			i. Pattern (soil, desert, rock)
			ii. Mask (desert, rock)
	4. FOOTER (Variable return statement)
*/
function main() {
	// First, we put Header String
    output += headerString(); 

    // Loop to get each layer on adobe file
    for (var i = 0, layersLength = layers.length; i < layersLength; i++) {
        var mapGroup = layers[i]; 
		layerName = mapGroup.name; 

        // Always skip invisible layer/group
        if (mapGroup.visible == false) continue; 

        // If the layer called start position, record it as start position output
        if (layerName.indexOf("start-position") == 0) {
			getLayerInfo(mapGroup);
        	output += addStartPosition(layerName, pos_x, pos_y); 

        } else {
			// section name is this group name 
			section = "\"" + mapGroup.name + "\""; 
		
	        // loop to get section 
	        var groupLayers = mapGroup.layers; 

	        for (var j = 0; j < groupLayers.length; j++) {
	        	var insideSection = groupLayers[j]; 
	        	layerName = insideSection.name; 

	        	if (insideSection.visible == false) continue; 

	        	if (layerName.indexOf("section-area") == 0) {
	        		//Getting section scale
	        		var idx 	= layerName.indexOf("_");
	        		var scale 	= 1

	        		if(idx != -1){
	        			scale = layerName.substring(idx + 1);	
	        		}
    				
					// Layer Group: section-area
	                // Write to the prime output string
	        		getLayerInfo(insideSection); 
	                output += addPortalArea(mapGroup.name, tx, ty, res_width, res_height, scale);
					output += "\nmap.portal[" + section + "] = {}"; 

	        	} else if (layerName.indexOf("default") == 0) {
	        		readInsideGroup(insideSection, mapGroup);
				} else if (layerName.indexOf("comp") == 0) {
					getLayerInfo(insideSection); 
	        		output += addComposition(layerName, tx, ty, res_width, res_height);
				
				} else if (layerName.indexOf("first-") == 0) {
					layerName = layerName.replace("first-", "") 
					var idx = layerName.indexOf("_");
					appearOrder = layerName.substring(0, idx); 
					layerName = layerName.substring(idx + 1); 
					
					layerName = layerName.split("_", 2);
					if (appearOrder.indexOf("appear") == 0) {
						appearOrder = "appearOrder = \"" + appearOrder + "\", appearOn = \"" + layerName[0] + "\", disappearOn = \"" + layerName[1] + "\", ";
					} else if (appearOrder.indexOf("disappear") == 0) {
						appearOrder = "appearOrder = \"" + appearOrder + "\", appearOn = \"" + layerName[1] + "\", disappearOn = \"" + layerName[0] + "\", ";
					} 
					
					readInsideGroup(insideSection, mapGroup); 
					appearOrder = ""; // reset appearOrder after read group inside it 

	        	} else if (layerName.indexOf("appear") == 0) {
					var idx = layerName.indexOf("_"); 
					appearOrder = "appearOn = \"" + layerName.substring(idx + 1) + "\", "; 
	        		readInsideGroup(insideSection, mapGroup); 
					appearOrder = ""; // reset appearOrder after read group inside it 

	        	} else if (layerName.indexOf("disappear") == 0) {
					var idx = layerName.indexOf("_"); 
					appearOrder = "disappearOn = \"" + layerName.substring(idx + 1) + "\", "; 
	        		readInsideGroup(insideSection, mapGroup); 
					appearOrder = ""; // reset appearOrder after read group inside it 

	        	} else if (layerName.indexOf("background") == 0) {
					// Layer Group: background
					// Variables for output text
					var bgPattern 	= "pattern = { "; 
					var bgMask 		= "mask = { "; 
				
					// Looping for each layer in this background group
					var insideBackground = insideSection.layers;
	                for (var k=insideBackground.length-1; k >= 0; k--) {
	                    var bgLayer = insideBackground[k]; 

	                    // Always skip invisible layer/group
	                    if (bgLayer.visible == false) continue; 

	                    // Keep layer info
	                    getLayerInfo(bgLayer) 

	                    // Mask layer
	                    if (layerName.indexOf("masking") != -1) {
	                        bgMask 		+= "\n\t\t" + addBgMask(layerName, tx, ty, res_width, res_height) + ", ";

	                    // Pattern layer
	                    } else {
	                        bgPattern 	+= "\n\t\t" + addBackground(layerName, tx, ty, res_width, res_height) + ", ";
	                    }
	                }
					
					// Output text closure
					bgPattern 	= bgPattern + "}, ";
					bgMask 		= bgMask + "}, ";
					
					// Write to the prime output string
					output 		+= addSectionBackground(mapGroup.name, bgPattern, bgMask); 

	        	}
	        }
        }
    }

    // Write footer string
    output += footerString();  

    // Finally, write the output string to file
    saveFile();
}

function readInsideGroup(insideSection, mapGroup) {
	// Loop for each data/layer inside this inside section group
	var insideSectionLayers = insideSection.layers; 
	for (var i = 0, insideSectionLayersLength = insideSectionLayers.length; i < insideSectionLayersLength; i++) {
		var objGroup = insideSectionLayers[i]; 
		layerName = objGroup.name; 

        // Always skip invisible layer/group
        if (objGroup.visible == false) continue;

        if (layerName == "top" || layerName == "upper" || layerName == "sorted" || layerName == "center" || layerName == "bottom" || layerName == "bottomPar" || layerName == "topPar") {
        	// Layer Group: top, upper, sorted, center, bottom
			orderLayer = layerName;

			var insideGroup = objGroup.layers; 
			for (var j=insideGroup.length-1; j >= 0; j--) {
				var objLayer = insideGroup[j]; 

				if (objLayer.typename == "LayerSet") {
					if (objLayer.name.indexOf("noon") == 0) {
						dayPreference = "dayPreference = \"noon\", "; 
						readObjectLayer(objLayer, mapGroup); 
					} else if (objLayer.name.indexOf("night") == 0) {
						dayPreference = "dayPreference = \"night\", "; 
						readObjectLayer(objLayer, mapGroup); 
					}
				}
			}

			dayPreference = "dayPreference = nil, "; 
			readObjectLayer(objGroup, mapGroup);

        } else if (layerName == "portal") {
			// Layer Group: Portal
			var insideGroup = objGroup.layers; 
			for (var j=insideGroup.length-1; j >= 0; j--) {
				var objLayer = insideGroup[j]; 

				if (objLayer.name.indexOf("noon") == 0) {
					dayPreference = "dayPreference = \"noon\", "; 
					readPortalLayer(objLayer, mapGroup); 
				} else if (objLayer.name.indexOf("night") == 0) {
					dayPreference = "dayPreference = \"night\", ";
					readPortalLayer(objLayer, mapGroup); 
				} 
			}

			dayPreference = "dayPreference = nil, "; 
			readPortalLayer(objGroup, mapGroup); 
        }
	}
}

function readPortalLayer(objGroup, mapGroup) { 
	var insideGroup = objGroup.layers; 
	for (var j = insideGroup.length-1; j >= 0; j--) {
		var portalLayer = insideGroup[j]; 

		// Always skip invisible layer/group
		if (portalLayer.visible == false) continue; 

		if (portalLayer.name.indexOf("noon") == 0 || portalLayer.name.indexOf("night") == 0) {
			continue; 
		} else {
			// Write to the prime output string
			output += addPortal(portalLayer, mapGroup.name); 
		}
		
	}
}

function readObjectLayer(objGroup, mapGroup) {
	var insideGroup = objGroup.layers; 
	for (var k=0, insideGroupLength=insideGroup.length; k < insideGroupLength; k++) {
		var objLayer = insideGroup[k];

		// Always skip invisible layer/group
		if (objLayer.visible == false) continue; 

		// Keep layer info
		getLayerInfo(objLayer); 

		// If we found a group
		if (objLayer.typename == "LayerSet") {
			// ENEMY TRIGGER 
			if (layerName.indexOf("trigger") == 0) {
				output += addEnemyTrigger(layerName, mapGroup.name, objLayer); 
				
			// SEARCHABLE 
			} else if (layerName.indexOf("searchable") == 0) { 
				output += addSearchable(layerName, objLayer); 
				
			// HIDDEN DIALOG 
			} else if (layerName.indexOf("hidden") == 0) {
				output += addHiddenDialog(objLayer); 

			// SUMMONGATE TRIGGER
			} else if(layerName.indexOf("gateTrigger") == 0) {
				output += addSummonGateTrigger(layerName, mapGroup.name, objLayer);

			} else if (layerName.indexOf("interactiveObject") == 0){
				output += addFolderedInteractiveObject(layerName, mapGroup.name, objLayer);
			
			} else if (layerName.indexOf("obstacle") == 0){
				output += addFolderedTrap(layerName, objLayer);
			}

		// Else if we found a layer
		} else {
			// ENEMY
			if (layerName.indexOf("enemy") == 0) {
				output += addEnemy(mapGroup.name, layerName, tx, ty, res_width, res_height);

			// NPC
			} else if (layerName.indexOf("npc") == 0) {
				output += addNPC(layerName, tx, ty, res_width, res_height); 

			// RANDOM OBJECT
			} else if (layerName.indexOf("random-object") == 0) {
				output += addObject("random-object", layerName, tx, ty, res_width, res_height);

			// ANIMATED OBJECT
			} else if (layerName.indexOf("animated-object") == 0) {
				output += addObject("animated-object", layerName, tx, ty, res_width, res_height);

			// DESERT OBJECT
			} else if (layerName.indexOf("lampu") == 0) {
				output += addObject("random-object", layerName, tx, ty, res_width, res_height); 

			// DESERT OBJECT
			} else if (layerName.indexOf("desert") == 0) {
				output += addObject("random-object", layerName, tx, ty, res_width, res_height); 

			// BUILDING
			} else if (layerName.indexOf("building") == 0) {
				output += addBuilding(layerName, tx, ty, res_width, res_height);

			// TRAP
			} else if (layerName.indexOf("trap") == 0) {
				output += addTrap(layerName, tx, ty, res_width, res_height);

			// OBSTACLE
			} else if (layerName.indexOf("obstacle") == 0) {
				output += addObstacle(layerName, pos_x, pos_y, res_width, res_height); 

			// QUEST TRIGGER
			} else if (layerName.indexOf("questTrigger") == 0) {
				output += addQuestTrigger(layerName, tx, ty, res_width, res_height); 

			// EVENT TRIGGER
			} else if (layerName.indexOf("eventTrigger") == 0) {
				output += addEventTrigger(layerName, tx, ty, res_width, res_height); 

			// SUMMON GATE
			} else if (layerName.indexOf("gate") == 0) { 
				output += addGate(mapGroup.name, layerName, tx, ty, res_width, res_height); 

			// BREAKABLE
			} else if(layerName.indexOf("break") == 0) {
				output += addBreakable(mapGroup.name, layerName, tx, ty, res_width, res_height);			

			} else if (layerName.indexOf("flee") == 0){
				output += addFleeingAnimal(layerName, tx, ty, res_width, res_height);	

			} else if (layerName.indexOf("triggerAnim") == 0){
				output += addTriggeredAnimatedObject(layerName, tx, ty, res_width, res_height);
					
			// ELSE... RANDOM OBJECT
			} else {
				output += addObject("random-object", layerName, tx, ty, res_width, res_height);
			}
		}
	}
}

// Run the main function
main();

/********************************* HELPER FUNCTIONS *********************************/
function addPortalArea(portal, px, py, w, h, scale) {
    var idx = portal.indexOf("_");
    portal = portal.substring(idx + 1);

    return "\n\n\nmap.section[\"" + portal + "\"] = { x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", scale = " + scale + " }";
}

function addStartPosition(name, px, py) {
	var idx = name.indexOf("_"); 
	name 	= name.substring(idx + 1); 
	
    return "\nmap.startPortal = \"" + name + "\""  
        + "\nmap.startPositionX = " + px
        + "\nmap.startPositionY = " + py;
}

function addBackground(name, px, py, w, h) {
	var idx = name.indexOf("_");
	name = name.substring(idx + 1);
    return "{ name = \"" + name + "\", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", }";
}

function addBgMask(name, px, py, w, h) {
	var idx = name.indexOf("_");
	name = name.substring(idx + 1);
    return "{ name = \"" + name + "\", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", }";
}

function addSectionBackground(portal, bgPattern, bgMask) { 
	var idx = portal.indexOf("_"); 
	portal = portal.substring(idx + 1); 
	return "\nmap.background[\"" + portal + "\"] = { \n\t" + bgPattern + "\n\t" + bgMask + "}";
}

function addPortal(portalGroup, sectionName) {
	var target = portalGroup.name; 
    var idx = target.indexOf("_"); 
	var type = target.substring(idx + 1); 
	var portal = "nil"; 
	var section = "nil"; 
	var event = "nil"; 
	target = target.substring(0, idx); 
	
	idx = type.indexOf("_event"); // find if there is event config 
	if (idx != -1) {
		event = "\"" + type.substring(idx + 7) + "\""; // get event from the config 
		type = type.substring(0, idx); // remove "_event_xxx" 
	}
	
	idx = type.indexOf("_portal"); // find if there is portal config 
	if (idx != -1) {
		portal = "\"" + type.substring(idx + 8) + "\""; // get portal from the config 
		type = type.substring(0, idx); // remove "_portal_xxx" 
	} 
	
	idx = type.indexOf("_section"); // find if there is section config 
	if (idx != -1) { 
		section = "\"" + type.substring(idx + 9) + "\""; // get section from the config 
		type = type.substring(0, idx); // remove "_section_xxx" 
	} 
	
	var centerX, centerY, px, py, w, h; 
	var rotation = "0"; 
	var open = "nil"; 
	var close = "nil"; 
	var balloon = "nil"; 
	var portals = ""; 


	var insidePortal = portalGroup.layers; 
	for (var j=insidePortal.length-1; j >= 0; j--) {
		var portalSet = insidePortal[j].layers; 
		var portalID = insidePortal[j].name; 

		for (var k=portalSet.length-1; k >=0; k--) {
			var portalObject = portalSet[k]; 
			
			getLayerInfo(portalObject); 
			
			if (portalObject.visible == false) continue; 
			
			if (layerName.indexOf("collision") == 0) {	
				centerX = pos_x; 
				centerY = pos_y; 
				px = tx; 
				py = ty; 
				w = res_width;
				h = res_height; 
				
				idx = layerName.indexOf("_rotation"); // find if there is rotation config 
				if (idx != -1) { 
					rotation = layerName.substring(idx + 10); // get rotation from the config 
					layerName = layerName.substring(0, idx); // remove "_rotation_xxx" 
				}
				
				idx = layerName.indexOf("_h"); // find if there is height config 
				if (idx != -1) {
					h = layerName.substring(idx + 3); // get height from the config 
					layerName = layerName.substring(0, idx); // remove "_h_xxx" 
				}
				
				idx = layerName.indexOf("_w"); // find if there is width config 
				if (idx != -1) {
					w = layerName.substring(idx + 3); // get height from the config 
					layerName = layerName.substring(0, idx); // remove "_w_xxx" 
				} 
			} else if (layerName.indexOf("open") == 0) {
				idx = layerName.indexOf("_"); 
				layerName = layerName.substring(idx + 1); 
				
				idx = layerName.indexOf("_"); 
				var order = layerName.substring(0, idx); 
				
				layerName = "random-object_" + layerName.substring(idx + 1); 
				open = addObjectHelper("random-object", layerName, tx, ty, res_width, res_height, order); 
			} else if (layerName.indexOf("close") == 0) {
				idx = layerName.indexOf("_"); 
				layerName = layerName.substring(idx + 1); 
				
				idx = layerName.indexOf("_"); 
				var order = layerName.substring(0, idx); 
				
				layerName = "random-object_" + layerName.substring(idx + 1); 
				close = addObjectHelper("random-object", layerName, tx, ty, res_width, res_height, order); 
			} else if (layerName.indexOf("balloon") == 0) {
				balloon = "{ x = " + pos_x + ", y = " + pos_y + "}"; 
			} 

			portals += "\n\t[\"" + portalID + "\"] = { " + dayPreference + appearOrder + "type = \"" + type + "\", target = \"" + target + "\", section = " + section + ", portal = " + portal + ", eventTrigger = " + event + ", rotation = " + rotation + ", centerX = " + centerX + ", centerY = " + centerY + ", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", \n\t\topen = " + open + ", \n\t\tclose = " + close + ", }, "; 
		}
	} 
	
	return "\nmap.portal[\"" + sectionName + "\"][\"" + target + "\"] = { " + portals + "}";
} 

function addObstacle(shape, px, py, w, h) {
	var idx = shape.indexOf("_"); 
	shape = shape.substring(idx + 1); 
	if (shape == "circle") {
		var radius = Math.max(w, h) 
		return "\nmap.obstacle[#map.obstacle + 1] = { shape = \"circle\", x = " + px + ", y = " + py + ", radius = " + radius*0.5 + ", }";
	} else if (shape == "rectangle") { 
		return "\nmap.obstacle[#map.obstacle + 1] = { shape = \"rectangle\", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", }"; 
	} 
} 

function addBuilding(name, px, py, w, h) {
	var idx = name.indexOf("_"); 
	var tag = "nil"; 
	var relativeY = "nil"; 
	var scale = "nil"; 
	var movementSpeed = "1";
	var rotation = "0"; 
	var flipH = "false"; 
	var flipV = "false"; 
	
	name = name.substring(idx + 1); // remove "building"
	idx = name.indexOf("_flipH"); // find if there is flip horizontal config 
	if (idx != -1) {
		flipH = "true"; // set flipH var to true 
		name = name.substring(0, idx); // remove "_flipH" 
	}
	
	idx = name.indexOf("_flipV"); // find if there is flip vertical config 
	if (idx != -1) {
		flipV = "true"; // set flipV var to true 
		name = name.substring(0, idx); // remove "_flipV" 
	}
	
	idx = name.indexOf("_ms"); // find if there is rotation config 
	if (idx != -1) {
		movementSpeed = name.substring(idx + 4); // get rotation from the config 
		name = name.substring(0, idx); // remove "_rotation_xxx" 
	}
	
	idx = name.indexOf("_rotation"); // find if there is rotation config 
	if (idx != -1) {
		rotation = name.substring(idx + 10); // get rotation from the config 
		name = name.substring(0, idx); // remove "_rotation_xxx" 
	}
	
	idx = name.indexOf("_scale"); // find if there is scale config 
	if (idx != -1) {
		scale = name.substring(idx + 7); // get scale from the config 
		name = name.substring(0, idx); // remove "_scale_xxx" 
	}
	
	idx = name.indexOf("_relativeY"); // find if there is relativeY config 
	if (idx != -1) {
		relativeY = name.substring(idx + 11); // get relativeY from the config 
		name = name.substring(0, idx); // remove "_relativeY_xxx" 
	}
	
	idx = name.indexOf("_tag"); // find if there is tag config 
	if (idx != -1) {
		tag = "\"" + name.substring(idx + 5) + "\""; // get tag from the config 
		name = name.substring(0, idx); // remove "_tag_xxx" 
	}
	
    return "\nmap.building[#map.building + 1] = { order = \"" + orderLayer 
    	+ "\", name = \"" + name + "\", tag = " + tag + ", relativeY = " + relativeY  + ", movementSpeed = " + movementSpeed 
    	+ ", rotation = " + rotation + ", scale = " + scale + ", flipH = " + flipH 
    	+ ", flipV = " + flipV + ", x = " + px + ", y = " + py + ", width = " + w 
    	+ ", height = " + h + ", section = " + section + ", " + dayPreference + appearOrder + "}";
}

function addFolderedTrap (name, group) {
	var layers = group.layers;
	var idx = name.indexOf("_");
	var id = name.substring(idx+1);

	if (id == "stalagmite"){
		return addStalagmite(layers);
	}
}

function addStalagmite (layers) {

	var stalagmite = "";
	var shadow	= "";
	
	for(var i=0; i<layers.length; ++i){
		getLayerInfo(layers[i]);

		if(layerName.indexOf("stalagmite") == 0){

			relativeY = "nil"; 
			scale = "nil"; 
			rotation = "0"; 
			folder = "nil"; 
			flipH = "false"; 
			flipV = "false"; 
			noOverlay = "false"
			movementSpeed = "1";

			idx = layerName.indexOf("_"); 	
			layerName = layerName.substring(idx + 1);

			idx = layerName.indexOf("_");
			var folder = layerName.substring(0, idx);		
			layerName = layerName.substring(idx + 1);			

			idx = layerName.indexOf("_noOverlay_"); // find if there is sequence setting
			if (idx != -1) {
				noOverlay = layerName.substring(idx + 11); // get noOverlayValue
				layerName = layerName.substring(0, idx); // remove "_seq_" 
			} 

			idx = layerName.indexOf("_flipH"); // find if there is flip horizontal config 
			if (idx != -1) {
				flipH = "true"; // set flipH var to true 
				layerName = layerName.substring(0, idx); // remove "_flipH" 
			} 
			
			idx = layerName.indexOf("_flipV"); // find if there is flip vertical config 
			if (idx != -1) {
				flipV = "true"; // set flipV var to true 
				layerName = layerName.substring(0, idx); // remove "_flipV" 
			} 
			
			idx = layerName.indexOf("_ms"); // find if there is ms config 
			if (idx != -1) {
				movementSpeed = layerName.substring(idx + 4); // get ms from the config 
				layerName = layerName.substring(0, idx); // remove "_ms_xxx" 
			}

			idx = layerName.indexOf("_rotation"); // find if there is rotation config 
			if (idx != -1) {
				rotation = layerName.substring(idx + 10); // get rotation from the config 
				layerName = layerName.substring(0, idx); // remove "_rotation_xxx" 
			}
			
			idx = layerName.indexOf("_scale"); // find if there is scale config 
			if (idx != -1) {
				scale = layerName.substring(idx + 7); // get scale from the config 
				layerName = layerName.substring(0, idx); // remove "_scale_xxx" 
			}
			
			idx = layerName.indexOf("_relativeY"); // find if there is relativeY config 
			if (idx != -1) {
				relativeY = layerName.substring(idx + 11); // get relativeY from the config 
				layerName = layerName.substring(0, idx); // remove "_relativeY_xxx" 
			} 			

			stalagmite = "{ type = \"random-object\", folder = \"" + folder + "\", name = \"" + layerName + "\" , relativeY = " + relativeY + ", movementSpeed = " + movementSpeed 
					+ ", rotation = " + rotation + ", scale = " + scale + ", noOverlay = " + noOverlay + ", flipH = " + flipH  
					+ ", flipV = " + flipV + ", x = " + tx + ", y = " + ty + ", width = " + res_width 
					+ ", height = " + res_height + "}";

		} else if (layerName.indexOf("shadow") == 0){

			relativeY = "nil"; 
			scale = "nil"; 
			rotation = "0"; 
			folder = "nil"; 
			flipH = "false"; 
			flipV = "false"; 
			noOverlay = "false"
			movementSpeed = "1";

			idx = layerName.indexOf("_"); 	
			layerName = layerName.substring(idx + 1);

			idx = layerName.indexOf("_");
			var folder = layerName.substring(0, idx);		
			layerName = layerName.substring(idx + 1);			

			idx = layerName.indexOf("_noOverlay_"); // find if there is sequence setting
			if (idx != -1) {
				noOverlay = layerName.substring(idx + 11); // get noOverlayValue
				layerName = layerName.substring(0, idx); // remove "_seq_" 
			} 

			idx = layerName.indexOf("_flipH"); // find if there is flip horizontal config 
			if (idx != -1) {
				flipH = "true"; // set flipH var to true 
				layerName = layerName.substring(0, idx); // remove "_flipH" 
			} 
			
			idx = layerName.indexOf("_flipV"); // find if there is flip vertical config 
			if (idx != -1) {
				flipV = "true"; // set flipV var to true 
				layerName = layerName.substring(0, idx); // remove "_flipV" 
			} 
			
			idx = layerName.indexOf("_ms"); // find if there is ms config 
			if (idx != -1) {
				movementSpeed = layerName.substring(idx + 4); // get ms from the config 
				layerName = layerName.substring(0, idx); // remove "_ms_xxx" 
			}

			idx = layerName.indexOf("_rotation"); // find if there is rotation config 
			if (idx != -1) {
				rotation = layerName.substring(idx + 10); // get rotation from the config 
				layerName = layerName.substring(0, idx); // remove "_rotation_xxx" 
			}
			
			idx = layerName.indexOf("_scale"); // find if there is scale config 
			if (idx != -1) {
				scale = layerName.substring(idx + 7); // get scale from the config 
				layerName = layerName.substring(0, idx); // remove "_scale_xxx" 
			}
			
			idx = layerName.indexOf("_relativeY"); // find if there is relativeY config 
			if (idx != -1) {
				relativeY = layerName.substring(idx + 11); // get relativeY from the config 
				layerName = layerName.substring(0, idx); // remove "_relativeY_xxx" 
			} 			

			shadow = "{ type = \"random-object\", folder = \"" + folder + "\", name = \"" + layerName + "\" , relativeY = " + relativeY + ", movementSpeed = " + movementSpeed 
					+ ", rotation = " + rotation + ", scale = " + scale + ", noOverlay = " + noOverlay + ", flipH = " + flipH  
					+ ", flipV = " + flipV + ", x = " + tx + ", y = " + ty + ", width = " + res_width 
					+ ", height = " + res_height + "}";

		}
	}

	return "\nmap.trap[#map.trap + 1] = { type = \"obstacle\", order = \"" + orderLayer + "\", name = \"stalagmite\", section = " 
			+ section + ", \n    stalagmite = " + stalagmite + ",\n    shadow = " + shadow + ", " + dayPreference + appearOrder + "}";
}

function addTrap(name, px, py, w, h) {
    var idx = name.indexOf("_");
    name = name.substring(idx + 1);

    if (name.indexOf("poisoned-puddle") == 0){
    	var idx = name.indexOf("_");
	    name = name.substring(idx + 1);

	    var idx = name.indexOf("_");
	    var puddleID = name;
	    var noOverlay = "true";
	    
	    if (idx > 0){
			puddleID = name.substring(0, idx-1);
    		noOverlay = name.substring(idx+1);
	    }
    	
    	return "\nmap.trap[#map.trap + 1] = { type = \"obstacle\", order = \"" + orderLayer + "\", name = \"" + "poisoned-puddle" + "\" , id = \"" + puddleID + "\", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", noOverlay =" + noOverlay + ", section = " + section + ", " + dayPreference + appearOrder + "}"; 
    }

    return "\nmap.trap[#map.trap + 1] = { type = \"obstacle\", order = \"" + orderLayer + "\", name = \"" + name + "\", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", section = " + section + ", " + dayPreference + appearOrder + "}";
       
}

function addComposition(name, px, py, w, h) {
    var idx 		= null;
	var color 		= "nil"; 
	var period 		= "nil";
	// Remove "comp"
	idx 	= name.indexOf("_"); 
	name 	= name.substring(idx + 1); 

	idx 	= name.indexOf("_"); 
	if (idx != -1) {
		period 	= "\"" + name.substring(0, idx) + "\""; 	// period
		color 	= "\"" + name.substring(idx + 1)+ "\""; 	// color
	}

    return "\nmap.composition[#map.composition + 1] = { section = " + section + ", x = " + px + ", y = " + py + ", color = " 
    		+ color + ", width = " + w + ", height = " + h + ", period = " + period + "}";
}


function addObject(type, name, px, py, w, h) {
    return "\nmap.object[#map.object + 1] = " + addObjectHelper(type, name, px, py, w, h); 
}

function addObjectHelper(type, name, px, py, w, h, orderConfig, typeConfig) {
	// format config: "random-object_<name of object>_relativeY_<number>_scale_<number in percent>_rotation_<number in degree> 
	var idx = name.indexOf("_");
	var tag = "nil"; 
	var relativeY = "nil"; 
	var scale = "nil"; 
	var rotation = "0"; 
	var folder = "nil"; 
	var flipH = "false"; 
	var flipV = "false"; 
	var sequence = "nil"; 
	var noOverlay = "false"
	var movementSpeed = "1";
	var order = orderLayer;
	var typeConf = "nil"; 
	var noParY = "false";

	if (orderConfig !== undefined && orderConfig != null) {
		order = orderConfig; 
	} 

	if (typeConfig !== undefined && typeConfig != null) {
		typeConf = typeConfig; 
	} 
	
	folder = name.substring(0, idx); 
	if (folder == "random-object" || folder == "animated-object") { 
		folder = "nil"; 
	} else {
		folder = "\"" + folder + "\""; 
	}
	
	name = name.substring(idx + 1); // remove "animated-object_" or "random_object"

	idx = name.indexOf("_noParY"); 
	if (idx != -1) {
		noParY = "true"; 
		name = name.substring(0, idx); 
	} 

	idx = name.indexOf("_seq_"); // find if there is sequence setting
	if (idx != -1) {
		sequence = "\"" + name.substring(idx + 5) + "\""; // get rotation from the config 
		name = name.substring(0, idx); // remove "_seq_" 
	} 

	idx = name.indexOf("_noOverlay_"); // find if there is sequence setting
	if (idx != -1) {
		noOverlay = name.substring(idx + 11); // get noOverlayValue
		name = name.substring(0, idx); // remove "_seq_" 
	} 

	idx = name.indexOf("_flipH"); // find if there is flip horizontal config 
	if (idx != -1) {
		flipH = "true"; // set flipH var to true 
		name = name.substring(0, idx); // remove "_flipH" 
	} 
	
	idx = name.indexOf("_flipV"); // find if there is flip vertical config 
	if (idx != -1) {
		flipV = "true"; // set flipV var to true 
		name = name.substring(0, idx); // remove "_flipV" 
	} 
	
	idx = name.indexOf("_ms"); // find if there is ms config 
	if (idx != -1) {
		movementSpeed = name.substring(idx + 4); // get ms from the config 
		name = name.substring(0, idx); // remove "_ms_xxx" 
	}

	idx = name.indexOf("_rotation"); // find if there is rotation config 
	if (idx != -1) {
		rotation = name.substring(idx + 10); // get rotation from the config 
		name = name.substring(0, idx); // remove "_rotation_xxx" 
	}
	
	idx = name.indexOf("_scale"); // find if there is scale config 
	if (idx != -1) {
		scale = name.substring(idx + 7); // get scale from the config 
		name = name.substring(0, idx); // remove "_scale_xxx" 
	}
	
	idx = name.indexOf("_relativeY"); // find if there is relativeY config 
	if (idx != -1) {
		relativeY = name.substring(idx + 11); // get relativeY from the config 
		name = name.substring(0, idx); // remove "_relativeY_xxx" 
	} 
	
	idx = name.indexOf("_tag"); // find if there is tag config 
	if (idx != -1) {
		tag = "\"" + name.substring(idx + 5) + "\""; // get tag from the config 
		name = name.substring(0, idx); // remove "_tag_xxx" 
	}

	return "{ order = \"" + order + "\", type = \""+ type + "\", name = \"" + name + "\", sequence = " + sequence
		+ ", folder = " + folder + ", tag = " + tag + ", relativeY = " + relativeY + ", movementSpeed = " + movementSpeed 
		+ ", rotation = " + rotation + ", scale = " + scale + ", noOverlay = " + noOverlay + ", flipH = " + flipH + ", noParY = " + noParY  
		+ ", flipV = " + flipV + ", x = " + px + ", y = " + py + ", width = " + w 
		+ ", height = " + h + ", section = " + section + ", " + dayPreference + appearOrder + "}";
} 

function addHiddenDialog(objLayer) {
	var result = ""; 
	var ballon = ""; 
	
	var insideHidden = objLayer.layers; 
	for (var j=insideHidden.length-1; j >= 0; j--) {
		var objHidden = insideHidden[j]; 
		
		getLayerInfo(objHidden); 
		
		if (objHidden.visible == false) continue; 
		
		if (layerName.indexOf("ballon") == 0) {
			ballon = "{ x = " + pos_x + ", y = " + pos_y + " }"; 
		} else if (layerName.indexOf("collision") == 0) {
			layerName = layerName.replace("collision", "random-object"); 
			result = addObject("hidden-dialog", layerName, pos_x, pos_y, res_width, res_height); 
		} 
	}
	
	return result.substr(0, result.length-1) + "ballon = " + ballon + ", " + result.substr(result.length-1);
}

function addSearchable(searchableName, objLayer) { 
	var idx, id;
	var items 	= ""; 
	var open 	= "nil"; 
	var close 	= "nil"; 
	var px 		= pos_x; 
	var py 		= pos_y; 
	var rubyID  = "nil";

	// find if there is rubyID
	idx = searchableName.indexOf("_rubyID");  
	if (idx != -1) {
		rubyID = "\"" + searchableName.substring(idx + 8) + "\"";
		searchableName 	= searchableName.substring(0, idx); 
	}

	idx	= searchableName.indexOf("_"); 
	id 	= searchableName.substring(idx + 1); 

	var insideSearchable = objLayer.layers; 
	for (var j=insideSearchable.length-1; j >= 0; j--) {
		var objSearchable = insideSearchable[j]; 
		
		getLayerInfo(objSearchable); 
		
		if (objSearchable.visible == false) continue; 
		
		if (layerName.indexOf("item") == 0) {
			idx = layerName.indexOf("_"); 
			var item = layerName.substring(idx + 1); 
			var num = 1; 
			
			idx = item.indexOf("_"); 
			if (idx != -1) { 
				num = item.substring(idx + 1); 
				item = item.substring(0, idx); 
			} 
			
			items = items + "{name = \"" + item + "\", num = " + num + "}, "; 
		} else if (layerName.indexOf("open") == 0) {
			idx = layerName.indexOf("_"); 
			layerName = "random-object_" + layerName.substring(idx + 1); 
			open = addObjectHelper("random-object", layerName, tx, ty, res_width, res_height); 
		} else if (layerName.indexOf("close") == 0) {
			idx = layerName.indexOf("_"); 
			layerName = "random-object_" + layerName.substring(idx + 1); 
			close = addObjectHelper("random-object", layerName, tx, ty, res_width, res_height); 
		} 
	} 
	
	return "\nmap.searchable[#map.searchable + 1] = { " + 
		appearOrder + "section = " + section + ", id = \"" + id + 
		"\", rubyID = " + rubyID + 
		", items = { " + items + "}, x = " + px + ", y = " + py + 
		", \n\topen = " + open + ", \n\tclose = " + close + ", }"; 
}

function addQuestTrigger(name, px, py, w, h) {
	var idx = name.indexOf("_"); 
	name = name.substring(idx + 1); 
	return "\nmap.questTrigger[#map.questTrigger + 1] = { " + appearOrder 
		+ "name = \"" + name + "\", x = " + px + ", y = " + py + ", width = " + w 
		+ ", height = " + h + ", section = " + section + ", }"; 
}

function addEventTrigger(name, px, py, w, h) {
	var idx = name.indexOf("_"); 
	name 	= name.substring(idx + 1); 
	
	idx = name.indexOf("_enemyRemainder"); 
	var enemyRemainder = "nil"; 
	if (idx != -1) {
		enemyRemainder = name.substring(idx + 16); 
		name = name.substring(0, idx); 
	} 
	idx = name.indexOf("_clear"); 
	var isClearDungeon = false; 
	if (idx != -1) {
		isClearDungeon = true
		name = name.substring(0, idx); 
	} 
	
	if (isClearDungeon == true) {
		return "\nmap.eventTrigger[#map.eventTrigger + 1] = { " + dayPreference + appearOrder + "eventTrigger = \"" + name + "\", section = " + section + ", type = \"clear\"}"; 
	} else if (enemyRemainder == "nil") {
		return "\nmap.eventTrigger[#map.eventTrigger + 1] = { " + dayPreference + appearOrder + "eventTrigger = \"" + name + "\", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", section = " + section + ", type = \"area\" }"; 
	} else {
		return "\nmap.eventTrigger[#map.eventTrigger + 1] = { " + dayPreference + appearOrder + "eventTrigger = \"" + name + "\", section = " + section + ", enemyRemainder = " + enemyRemainder + ", type = \"enemyRemainder\" }"; 
	} 
} 

function addFolderedInteractiveObject(name, portal, group){
	var layers = group.layers;
	var idx = name.indexOf("_"); 
	var id = "";

	name = name.substring(idx + 1); // remove "interactiveObject"

	idx = name.indexOf("_");

	if (idx >= 0){
		id = name.substring(0, idx);	
	} else{
		id = name;
	}
	
	if(id == "falling-pillar"){
		var triggerID = name.substring(idx + 1);
		return addFallingPillar(layers, triggerID);
	} else if (id == "steep-sand"){
		return addSteepSand(layers);
	} else if (id == "animal"){
		return addInteractiveAnimal(layers);
	} else if (id == "flee"){
		var animalName = name.substring(idx + 1);
		return addFolderedFleeingAnimal(layers, animalName);
	}
}

function addFolderedFleeingAnimal (layers, animalName) {
	
	var idx = 0;
	var trigger = "";
	var before = "";
	var after = "";
	var shadow = "";
	var animal = "";

	for(var i=0; i<layers.length; ++i){
		var fleeingAnimal = layers[i];
		if (fleeingAnimal.visible == false) continue;
		getLayerInfo(fleeingAnimal);

		idx = layerName.indexOf("_");

		if (layerName.indexOf("before") == 0) {

			var heading = "up-right"; 			
			
			idx = layerName.indexOf("_"); // find if there is heading config 
			if (idx != -1) {
				heading = layerName.substring(idx + 1); // get heading from the config 
			}

			var head1 = "check"; 
			var head2 = "check"; 
			idx = heading.indexOf("-"); 
			if (idx != -1) {
				head1 = heading.substring(0, idx); 
				head2 = heading.substring(idx + 1); 
			} else {
				head1 = heading; 
			}

			before = "{" + " x = " + tx + ", y = " + ty + ", heading = {\"" + head1 + "\", \"" + head2 + 
				"\"}," + " width = " + res_width + ", height = " + res_height + "}" ;
		} else if (layerName.indexOf("trigger") == 0) {
			trigger = "{" + " x = " + tx + ", y = " + ty + ", width = " + res_width + ", height = " + res_height + "}" ;
		} else if (layerName.indexOf("after") == 0) { 

			var heading = "up-right"; 			
			
			idx = layerName.indexOf("_"); // find if there is heading config 
			if (idx != -1) {
				heading = layerName.substring(idx + 1); // get heading from the config 
			}

			var head1 = "check"; 
			var head2 = "check"; 
			idx = heading.indexOf("-"); 
			if (idx != -1) {
				head1 = heading.substring(0, idx); 
				head2 = heading.substring(idx + 1); 
			} else {
				head1 = heading; 
			}

			after = "{" + " x = " + tx + ", y = " + ty + ", heading = {\"" + head1 + "\", \"" + head2 + 
				"\"}" + ", width = " + res_width + ", height = " + res_height + "}" ;
		} else if (layerName.indexOf("shadow") == 0) {
			shadow = "{" + " x = " + tx + ", y = " + ty + ", width = " + res_width + ", height = " + res_height + "}" ;
		} 
	}

	return "\nmap.object[#map.object + 1] = { order = \""+ orderLayer + 
    	"\", type = \"interactiveObject\", name = \"fleeing\", animalName = \"" + animalName 
    	+ "\", \n    shadow = " + shadow + ", \n    before = " + before + ", \n    after = " + after + 
    	", \n    trigger = " + trigger + ", section = " + section + ", " + dayPreference + appearOrder + "}";

}

function addFleeingAnimal (name, x,y,width,height) {
	var idx = name.indexOf("_");

	var heading = "right-front"; 
	var count = "1";
	
	name = name.substring(idx + 1); // remove id 

	idx = name.indexOf("_count"); // find if there is flip config 
	if (idx != -1) {
		count = name.substring(idx + 7); // get flip from the config
		name = name.substring(0, idx); // remove "_flip_xxx" 
	} 
	
	idx = name.indexOf("_heading"); // find if there is heading config 
	if (idx != -1) {
		heading = name.substring(idx + 9); // get heading from the config 
		name = name.substring(0, idx); // remove "_heading_xxx" 
	} 
	
	var head1 = "check"; 
	var head2 = "check"; 
	idx = heading.indexOf("-"); 
	if (idx != -1) {
		head1 = heading.substring(0, idx); 
		head2 = heading.substring(idx + 1); 
	} else {
		head1 = heading; 
	}
	
    return "\nmap.object[#map.object + 1] = { order = \""+ orderLayer + 
    	"\", type = \"interactiveObject\", count = " + count + ", name = \"fleeing\", animal = { name = \"" + name
    	+ "\", heading = {\"" + head1 + "\", \"" + head2 + "\"}, x = " + x + 
    	", y = " + y + ", width = " + width + ", height = " + height + "}, section = " + section + 
    	", " + dayPreference + appearOrder + "}";
}

function addTriggeredAnimatedObject (name, x,y,width,height) {
	var idx = name.indexOf("_");

	var triggered = "true";
	
	name = name.substring(idx + 1); // remove id 

	idx = name.indexOf("_auto"); // find if there is flip config 
	if (idx != -1) {
		triggered = "false"; // get flip from the config
		name = name.substring(0, idx); // remove "_flip_xxx" 
	} 
	
    return "\nmap.object[#map.object + 1] = { order = \""+ orderLayer + 
    	"\", type = \"interactiveObject\", triggered = " + triggered + ", name = \"triggerAnim\", object = \"" + name 
    	+ "\", x = " + x + 
    	", y = " + y + ", width = " + width + ", height = " + height + ", section = " + section + 
    	", " + dayPreference + appearOrder + "}";
}

function addInteractiveAnimal (layers) {
	
	var trigger = "nil";
	var relativeY = "nil"; 
	var scale = "nil"; 
	var rotation = "0"; 
	var folder = "nil"; 
	var flipH = "false"; 
	var flipV = "false";  
	var noOverlay = "false"
	var movementSpeed = "1";
	var animal = "nil";
	var shadow = "nil";

	for(var i=0; i<layers.length; ++i){
		var animalObject = layers[i];

		if (animalObject.visible == false) continue;

		getLayerInfo(animalObject);

		if (layerName.indexOf("trigger") == 0) {
			idx = layerName.indexOf("_"); 
			trigger = "{" + " x = " + tx + ", y = " + ty + ", width = " + res_width + ", height = " + res_height + "}" ;

		} else if (layerName.indexOf("animal") == 0) {
			relativeY = "nil"; 
			scale = "nil"; 
			rotation = "0"; 
			folder = "nil"; 
			flipH = "false"; 
			flipV = "false"; 
			noOverlay = "false"
			movementSpeed = "1";

			idx = layerName.indexOf("_"); 	
			layerName = layerName.substring(idx + 1);

			idx = layerName.indexOf("_");
			var folder = layerName.substring(0, idx);		
			layerName = layerName.substring(idx + 1);			

			idx = layerName.indexOf("_noOverlay_"); // find if there is sequence setting
			if (idx != -1) {
				noOverlay = layerName.substring(idx + 11); // get noOverlayValue
				layerName = layerName.substring(0, idx); // remove "_seq_" 
			} 

			idx = layerName.indexOf("_flipH"); // find if there is flip horizontal config 
			if (idx != -1) {
				flipH = "true"; // set flipH var to true 
				layerName = layerName.substring(0, idx); // remove "_flipH" 
			} 
			
			idx = layerName.indexOf("_flipV"); // find if there is flip vertical config 
			if (idx != -1) {
				flipV = "true"; // set flipV var to true 
				layerName = layerName.substring(0, idx); // remove "_flipV" 
			} 
			
			idx = layerName.indexOf("_ms"); // find if there is ms config 
			if (idx != -1) {
				movementSpeed = layerName.substring(idx + 4); // get ms from the config 
				layerName = layerName.substring(0, idx); // remove "_ms_xxx" 
			}

			idx = layerName.indexOf("_rotation"); // find if there is rotation config 
			if (idx != -1) {
				rotation = layerName.substring(idx + 10); // get rotation from the config 
				layerName = layerName.substring(0, idx); // remove "_rotation_xxx" 
			}
			
			idx = layerName.indexOf("_scale"); // find if there is scale config 
			if (idx != -1) {
				scale = layerName.substring(idx + 7); // get scale from the config 
				layerName = layerName.substring(0, idx); // remove "_scale_xxx" 
			}
			
			idx = layerName.indexOf("_relativeY"); // find if there is relativeY config 
			if (idx != -1) {
				relativeY = layerName.substring(idx + 11); // get relativeY from the config 
				layerName = layerName.substring(0, idx); // remove "_relativeY_xxx" 
			} 			

			animal = "{ name = \"" + layerName + "\" , relativeY = " + relativeY + ", movementSpeed = " + movementSpeed 
					+ ", rotation = " + rotation + ", scale = " + scale + ", noOverlay = " + noOverlay + ", flipH = " + flipH  
					+ ", flipV = " + flipV + ", x = " + tx + ", y = " + ty + ", width = " + res_width 
					+ ", height = " + res_height + "}";

		} else if (layerName.indexOf("shadow") == 0) {
			shadow = "{" + " x = " + tx + ", y = " + ty + ", width = " + res_width + ", height = " + res_height + "}" ;
		}

	}

	return "\nmap.object[#map.object + 1] = " + "{ type = \"interactiveObject\"" + ", name = \"animal\", section = " + section + ", " + dayPreference + appearOrder 
			+ "\n    trigger = " + trigger + ", \n    animal = " + animal + ", \n    shadow = " + shadow + "}";

}

function addSteepSand(layers){

	var steepSands = "";
	var direction = "";
	var force = "";
	var rotation = "";
	var width = "";
	var height = "";
	var idx = 0;

	for(var i=0; i<layers.length; ++i){
		var steepSand = layers[i];
		if (steepSand.visible == false) continue;
		getLayerInfo(steepSand);

		idx = layerName.indexOf("_rotation"); // find if there is sequence setting
		if (idx != -1) {
			rotation = layerName.substring(idx + 10); // get noOverlayValue
			layerName = layerName.substring(0, idx); // remove "_seq_" 

			idx = layerName.indexOf("_h"); // find if there is flip horizontal config 
			height = layerName.substring(idx + 3); // get noOverlayValue
			layerName = layerName.substring(0, idx);

			idx = layerName.indexOf("_w"); // find if there is sequence setting
			width = layerName.substring(idx + 3); // get noOverlayValue
			layerName = layerName.substring(0, idx); // remove "_seq_" 			
		} else{
			rotation = "0";
			height = res_height;
			width = res_width;
		}		

		idx = layerName.indexOf("_force");
		force = layerName.substring(idx + 7);
		layerName = layerName.substring(0, idx);
		
		idx = layerName.indexOf("dir_");
		direction = layerName.substring(idx + 4);
		layerName = layerName.substring(0, idx);

		steepSands += "\nmap.object[#map.object + 1] = " + "{ order = \"" + orderLayer 
						+ "\", type = \"interactiveObject\"" + ", name = \"steep-sand\", section = " 
						+ section + ", " + dayPreference + appearOrder + 
						" direction = \"" + direction + "\", force = " + force + ", centerX = " + pos_x + ", centerY = " + pos_y + ", x = " + tx + 
						", y = " + ty + ", width = " + width + ", height = " + height + ", rotation = " + rotation + "}";
	}

	return steepSands;

}

function addFallingPillar(layers, triggerID){

	var trigger = "nil";
	var relativeY = "nil"; 
	var scale = "nil"; 
	var rotation = "0"; 
	var folder = "nil"; 
	var flipH = "false"; 
	var flipV = "false";  
	var noOverlay = "false"
	var movementSpeed = "1";
	var before = "nil";
	var after = "nil";
	var config = "nil";

	for(var i=0; i<layers.length; ++i){
		var pillarObject = layers[i];

		if (pillarObject.visible == false) continue;

		getLayerInfo(pillarObject);

		if (layerName.indexOf("trigger") == 0) {
			idx = layerName.indexOf("_"); 
			trigger = "{" + " x = " + tx + ", y = " + ty + ", width = " + res_width + ", height = " + res_height + "}" ;

		} else if (layerName.indexOf("before") == 0) {
			relativeY = "nil"; 
			scale = "nil"; 
			rotation = "0"; 
			folder = "nil"; 
			flipH = "false"; 
			flipV = "false"; 
			noOverlay = "false"
			movementSpeed = "1";

			idx = layerName.indexOf("_"); 	
			layerName = layerName.substring(idx + 1);

			idx = layerName.indexOf("_");
			var folder = layerName.substring(0, idx);		
			layerName = layerName.substring(idx + 1);			

			idx = layerName.indexOf("_noOverlay_"); // find if there is sequence setting
			if (idx != -1) {
				noOverlay = layerName.substring(idx + 11); // get noOverlayValue
				layerName = layerName.substring(0, idx); // remove "_seq_" 
			} 

			idx = layerName.indexOf("_flipH"); // find if there is flip horizontal config 
			if (idx != -1) {
				flipH = "true"; // set flipH var to true 
				layerName = layerName.substring(0, idx); // remove "_flipH" 
			} 
			
			idx = layerName.indexOf("_flipV"); // find if there is flip vertical config 
			if (idx != -1) {
				flipV = "true"; // set flipV var to true 
				layerName = layerName.substring(0, idx); // remove "_flipV" 
			} 
			
			idx = layerName.indexOf("_ms"); // find if there is ms config 
			if (idx != -1) {
				movementSpeed = layerName.substring(idx + 4); // get ms from the config 
				layerName = layerName.substring(0, idx); // remove "_ms_xxx" 
			}

			idx = layerName.indexOf("_rotation"); // find if there is rotation config 
			if (idx != -1) {
				rotation = layerName.substring(idx + 10); // get rotation from the config 
				layerName = layerName.substring(0, idx); // remove "_rotation_xxx" 
			}
			
			idx = layerName.indexOf("_scale"); // find if there is scale config 
			if (idx != -1) {
				scale = layerName.substring(idx + 7); // get scale from the config 
				layerName = layerName.substring(0, idx); // remove "_scale_xxx" 
			}
			
			idx = layerName.indexOf("_relativeY"); // find if there is relativeY config 
			if (idx != -1) {
				relativeY = layerName.substring(idx + 11); // get relativeY from the config 
				layerName = layerName.substring(0, idx); // remove "_relativeY_xxx" 
			} 			

			before = "{ type = \"random-object\", folder = \"" + folder + "\", name = \"" + layerName + "\" , relativeY = " + relativeY + ", movementSpeed = " + movementSpeed 
					+ ", rotation = " + rotation + ", scale = " + scale + ", noOverlay = " + noOverlay + ", flipH = " + flipH  
					+ ", flipV = " + flipV + ", x = " + tx + ", y = " + ty + ", width = " + res_width 
					+ ", height = " + res_height + "}";

		} else if (layerName.indexOf("after") == 0) {
			relativeY = "nil"; 
			scale = "nil"; 
			rotation = "0"; 
			folder = "nil"; 
			flipH = "false"; 
			flipV = "false"; 
			noOverlay = "false"
			movementSpeed = "1";

			idx = layerName.indexOf("_"); 			
			layerName = layerName.substring(idx + 1);

			idx = layerName.indexOf("_");
			var folder = layerName.substring(0, idx);		
			layerName = layerName.substring(idx + 1);

			idx = layerName.indexOf("_noOverlay_"); // find if there is sequence setting
			if (idx != -1) {
				noOverlay = layerName.substring(idx + 11); // get noOverlayValue
				layerName = layerName.substring(0, idx); // remove "_seq_" 
			} 

			idx = layerName.indexOf("_flipH"); // find if there is flip horizontal config 
			if (idx != -1) {
				flipH = "true"; // set flipH var to true 
				layerName = layerName.substring(0, idx); // remove "_flipH" 
			} 
			
			idx = layerName.indexOf("_flipV"); // find if there is flip vertical config 
			if (idx != -1) {
				flipV = "true"; // set flipV var to true 
				layerName = layerName.substring(0, idx); // remove "_flipV" 
			} 
			
			idx = layerName.indexOf("_ms"); // find if there is ms config 
			if (idx != -1) {
				movementSpeed = layerName.substring(idx + 4); // get ms from the config 
				layerName = layerName.substring(0, idx); // remove "_ms_xxx" 
			}

			idx = layerName.indexOf("_rotation"); // find if there is rotation config 
			if (idx != -1) {
				rotation = layerName.substring(idx + 10); // get rotation from the config 
				layerName = layerName.substring(0, idx); // remove "_rotation_xxx" 
			}
			
			idx = layerName.indexOf("_scale"); // find if there is scale config 
			if (idx != -1) {
				scale = layerName.substring(idx + 7); // get scale from the config 
				layerName = layerName.substring(0, idx); // remove "_scale_xxx" 
			}
			
			idx = layerName.indexOf("_relativeY"); // find if there is relativeY config 
			if (idx != -1) {
				relativeY = layerName.substring(idx + 11); // get relativeY from the config 
				layerName = layerName.substring(0, idx); // remove "_relativeY_xxx" 
			} 			

			after = "{ type = \"random-object\", folder = \"" + folder + "\", name = \"" + layerName + "\" , relativeY = " + relativeY + ", movementSpeed = " + movementSpeed 
					+ ", rotation = " + rotation + ", scale = " + scale + ", noOverlay = " + noOverlay + ", flipH = " + flipH  
					+ ", flipV = " + flipV + ", x = " + tx + ", y = " + ty + ", width = " + res_width 
					+ ", height = " + res_height + "}";
		}

	}

	config = "{ triggerID = " + triggerID + ", trigger = " + trigger + ", \n    before = " + before + ", \n    after = " + after + "}";

	return "\nmap.object[#map.object + 1] = " + "{ order = \"" + orderLayer + "\", type = \"interactiveObject\"" + ", name = \"falling-pillar\", section = " + section + ", " + dayPreference + appearOrder + "\n    config = " + config + "}";
}

function addSummonGateTrigger(name, portal, group){
	var triggerLayer = group.layers;
	var gates = "gates = { ";
	var idx = 0;
	var area = "nil";
	var fromLastSpawn = "nil";
	var enemyRemainder = "nil";

	for (var i=0, sum=triggerLayer.length; i < sum; i++) {
		var gate = triggerLayer[i];

		if (gate.visible == false) continue;

		getLayerInfo(gate);

		if (layerName.indexOf("area") != -1) {
			area = "{" + tx + "," + ty + "," + res_width + "," + res_height + "}"; 
		} else if (layerName.indexOf("lastSpawn") != -1) { 
			idx = layerName.indexOf("_"); 
			fromLastSpawn = layerName.substring(idx + 1); 
		} else if (layerName.indexOf("remainder") != -1) {
			idx = layerName.indexOf("_"); 
			enemyRemainder = layerName.substring(idx + 1); 
		} else if (layerName.indexOf("gate") != -1){
			var tmp = addSummonGateOnTrigger(portal, layerName, tx, ty, res_width, res_height);  			
			gates += "\n\t" + "{ " + tmp + ", }, ";
		}
	}

	var prev = "nil"; 
	idx = name.indexOf("_"); 
	name = name.substring(idx + 1); // remove "trigger" 
	idx = name.indexOf("_prev"); // find if there is prev config 
	if (idx != -1) {
		prev = "\"" + name.substring(idx + 6) + "\""; // get prev from the config 
		name = name.substring(0, idx); // remove "_prev_xxx" 
	} 

	return "\nmap.gateTrigger[#map.gateTrigger + 1] = { section = \"" + portal 
		+ "\", " + dayPreference + appearOrder + "triggerID = \"" + name + "\", prev = " + prev + ", area = " + area 
		+ ", fromLastSpawn = " + fromLastSpawn 
		+ ", enemyRemainder = " + enemyRemainder + ", " + gates + "}, }";	

}

function addEnemyTrigger(name, portal, group) {
    var triggerLayer = group.layers; 
	var enemies = "enemies = { "; 
	var idx = 0; 
	var area = "nil"; 
	var triggerApproach = "false"; 
	var fromLastSpawn = "nil"; 
	var enemyRemainder = "nil"; 
	var bgm = "nil";

	for (var i=0, sum=triggerLayer.length; i < sum; i++) {
		var enemy = triggerLayer[i] 
		if (enemy.visible == false) continue; 
		
		getLayerInfo(enemy);  
		
		if (layerName.indexOf("enemy") != -1) {
			var tmp = addEnemyOnTrigger(portal, layerName, tx, ty, res_width, res_height); 
			var approach = "nil"; 
	
			if (i + 1 < sum) { 
				getLayerInfo(triggerLayer[i + 1]); 
				
				if (layerName.indexOf("approach") != -1) {
					triggerApproach = "true"; 
					approach = "{ x = " + pos_x + ", y = " + pos_y + ", }"; 
					i = i + 1;				
				} 
			} 
			
			enemies += "\n\t" + "{ approach = " + approach + ", " + tmp + ", }, ";
		} else if (layerName.indexOf("area") != -1) {
			area = "{" + tx + "," + ty + "," + res_width + "," + res_height + "}"; 
			if( layerName.indexOf("_bgm_") != -1 ) {
				idx = layerName.indexOf("_bgm_");
				bgm = "\"" + layerName.substring(idx + 5) + "\"";
			}
			fromLastSpawn = layerName.substring(idx + 1); 
		} else if (layerName.indexOf("lastSpawn") != -1) { 
			idx = layerName.indexOf("_"); 
			fromLastSpawn = layerName.substring(idx + 1); 
		} else if (layerName.indexOf("remainder") != -1) {
			idx = layerName.indexOf("_"); 
			enemyRemainder = layerName.substring(idx + 1); 
		} 
	}
	
	var prev = "nil"; 
	idx = name.indexOf("_"); 
	name = name.substring(idx + 1); // remove "trigger" 
	idx = name.indexOf("_prev"); // find if there is prev config 
	if (idx != -1) {
		prev = "\"" + name.substring(idx + 6) + "\""; // get prev from the config 
		name = name.substring(0, idx); // remove "_prev_xxx" 
	} 
	
	return "\nmap.enemyTrigger[#map.enemyTrigger + 1] = { section = \"" + portal 
		+ "\", " + dayPreference + appearOrder + "triggerID = \"" + name + "\", prev = " + prev + ", area = " + area 
		+ ", approach = " + triggerApproach + ", fromLastSpawn = " + fromLastSpawn 
		+ ", enemyRemainder = " + enemyRemainder + ", bgm = " + bgm + ", " + enemies + "}, }";
}

function addEnemy(portal, name, px, py, w, h) {
    return "\nmap.enemy[#map.enemy + 1] = { " + dayPreference + appearOrder + addEnemyOnTrigger(portal, name, px, py, w, h) + ", }";
}

function addBreakable(portal, layerName, px, py, w, h){

	var idx = layerName.indexOf("_");
	var data = "data = {"; 
    
	layerName = layerName.substring(idx + 1); // remove "break_" 

	idx = layerName.indexOf("_");
	var name = layerName.substring(0,idx); // get name
	layerName = layerName.substring(idx + 1); // remove name

	idx = layerName.indexOf("_");
	layerName = layerName.substring(idx + 1); // remove _type

	if (idx = layerName.indexOf("item") == 0){
		layerName = layerName.substring(idx + 4); // remove _item_

		idx = layerName.indexOf("_");
		layerName = layerName.substring(idx + 1); // remove "id_"
		 		
		idx = layerName.indexOf("_");
		var id = layerName.substring(0,idx); // get id
		layerName = layerName.substring(idx + 1);

		idx = layerName.indexOf("_");
		var itemName = layerName.substring(0,idx); // get itemName
		layerName = layerName.substring(idx + 1);

		var itemCount = layerName;
		var itemDropCount = "1";		
		idx = layerName.indexOf("_");		
		if (idx > -1){
			itemCount = layerName.substring(0,idx);
			itemDropCount = layerName.substring(idx + 1); // get itemDropCount			
		}


		data += "type = \"item\", id = " + id + ", itemName = \"" + itemName + "\", itemCount = " + itemCount 
			+ ", itemDropCount = " + itemDropCount + ", }";

	} else if (idx = layerName.indexOf("enemy") == 0){
		layerName = layerName.substring(idx + 5); // remove _enemy_

		var minLevel = "nil"; 
		var maxLevel = "nil"; 
		var minRelativeLV = "nil"; 
		var maxRelativeLV = "nil";

		idx = layerName.indexOf("_");
		var dispatchEnemy = layerName.substring(0,idx); // get enemyName
		layerName = layerName.substring(idx + 1);

		idx = layerName.indexOf("_");
		var dispNum = layerName.substring(0,idx); // get enemyDispatchNum
		layerName = layerName.substring(idx + 1);		

		idx = layerName.indexOf("_maxLevel_"); // find if there is maxLevel config 
		if (idx != -1) {
			maxLevel = layerName.substring(idx + 10); // get maxLevel from the config 
			layerName = layerName.substring(0, idx); // remove "_maxLevel_xxx" 
		}
		
		idx = layerName.indexOf("minLevel_"); // find if there is minLevel config 
		if (idx != -1) {
			minLevel = layerName.substring(idx + 9); // get minLevel from the config 
			layerName = layerName.substring(0, idx); // remove "_minLevel_xxx" 
		}

		idx = layerName.indexOf("_maxRLevel_"); 			// find if there is maxRelativeLV config 
		if (idx != -1) {
			maxRelativeLV = layerName.substring(idx + 11); 	// get maxLevel from the config 
			layerName = layerName.substring(0, idx); 				// remove "_maxLevel_xxx" 
		}

		idx = layerName.indexOf("minRLevel_"); 			// find if there is minRelativeLV config 
		if (idx != -1) {
			minRelativeLV = layerName.substring(idx + 10); 	// get minLevel from the config 
			layerName = layerName.substring(0, idx); 				// remove "_minLevel_xxx" 
		}

		data += "type = \"enemy\", " + "dispatchEnemy = \"" + dispatchEnemy + "\", dispatchNum = {" + dispNum + "}, minRelativeLV = " + minRelativeLV 
		+ ", maxRelativeLV = " + maxRelativeLV + ", minLevel = " + minLevel + ", maxLevel = " + maxLevel + " }";

	} else if (idx = layerName.indexOf("gold") == 0){
		layerName = layerName.substring(idx + 4); // remove _gold_

		var goldRange = "";
		var itemDropCount = "1";
		var itemName = "nil";
		var itemCount = "nil";

		idx = layerName.indexOf("_");

		goldRange = layerName.substring(0,idx);
		layerName = layerName.substring(idx + 1);

		idx = layerName.indexOf("_");
		layerName = layerName.substring(idx + 1); // remove _count

		itemDropCount = layerName;

		idx = layerName.indexOf("_");

		if (idx != -1){

			itemDropCount = layerName.substring(0,idx);
			layerName = layerName.substring(idx + 1);

			idx = layerName.indexOf("_");
			layerName = layerName.substring(idx + 1); // remove item_

			idx = layerName.indexOf("_");
			itemName = "\"" + layerName.substring(0,idx) + "\"";
			itemCount = layerName.substring(idx + 1);
		}

		data += "type = \"gold\", goldRange = {" + goldRange + "}, itemDropCount = " + itemDropCount 
			+ ", itemName = " + itemName + ", itemCount = " + itemCount + "}";

	}
 
	return "\nmap.breakable[#map.breakable + 1] = { " + "section = \"" + portal + "\", name = \"" 
		+ name + "\"" + ", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ",\n " + data + "}"
}

function addGate(portal, data, px, py, w, h) {
	// format config: gate_<name of gate>_<gate data>_minLevel_<gate min level, default: 1>_maxLevel_<gate max level, default: 1> 
	var idx = data.indexOf("_"); 

	// 0 => gate, 1 => name of summon gate, 2 => gate minLevel, 3 => gate maxLevel, 4 => name of enemy, 5 => dispatchInterval, 6 => dispatchNum, 7 => maxNum, 8 => scale, 9 => isDestroyable, 10 => minLevel, 11 => maxLevel 
	data = data.split('_'); 
	var config = "{ dispatchEnemy = \"" + data[4] + "\", dispatchInterval = {" + data[5] + "}, dispatchNum = {" + data[6] + "}, maxNum = " + data[7] + ", scale = " + data[8] + ", isDestroyable = " + data[9] + ", minLevel = " + data[10] + ", maxLevel = " + data[11] + ", }" 
	
	return "\nmap.gate[#map.gate + 1] = { section = \"" + portal + "\", " + dayPreference + appearOrder + "name = \"" + data[1] + "\", minLevel = " + data[2] + ", maxLevel = " + data[3] + ", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", data = " + config + ", }";
}

function addSummonGateOnTrigger(portal, name, px, py, w, h){
	var idx = name.indexOf("_");
	var minLevel = "nil"; 
	var maxLevel = "nil";
	var minRelativeLV = "nil";
	var maxRelativeLV = "nil";
	var data = "";
	var dispatchEnemy = "";
	var dispatchNum = "";
	var dispatchInterval = "";
	var maxNum = "";
	var scale = "";
	var isDestroyable = "";
	var isToAddData = false;
	
	name = name.substring(idx + 1); // remove "gate"

	var isDestroy = parseInt(name.substring(name.length-1)) == 1;
	isDestroyable = "isDestroyable = " + isDestroy + ",";
	name = name.substring(0, name.length-2);	

	idx = name.indexOf("_xDisp"); 			// find if there is maxNum config 
	if (idx != -1) {
		isToAddData = true;
		maxNum = "maxNum = " + name.substring(idx + 7)+ ","; 	// get maxNum from the config 
		name = name.substring(0, idx); 				// remove "_maxNum_xxx" 
	}

	idx = name.indexOf("_disp"); 			// find if there is dispatchNum config 
	if (idx != -1) {
		isToAddData = true;
		dispatchNum = "dispatchNum = {" + name.substring(idx + 6)+ "},"; 	// get dispatchNum from the config 
		name = name.substring(0, idx); 				// remove "_dispatchNum_xxx" 
	}	

	idx = name.indexOf("_xL"); // find if there is maxLevel config 
	if (idx != -1) {
		maxLevel = name.substring(idx + 4); // get maxLevel from the config 
		name = name.substring(0, idx); // remove "_maxLevel_xxx" 
	}	
	
	idx = name.indexOf("_nL"); // find if there is minLevel config 
	if (idx != -1) {
		minLevel = name.substring(idx + 4); // get minLevel from the config 
		name = name.substring(0, idx); // remove "_minLevel_xxx" 
	}

	idx = name.indexOf("_xRL"); 			// find if there is maxRelativeLV config 
	if (idx != -1) {
		maxRelativeLV = name.substring(idx + 5); 	// get maxLevel from the config 
		name = name.substring(0, idx); 				// remove "_maxLevel_xxx" 
	}

	idx = name.indexOf("_nRL"); 			// find if there is minRelativeLV config 
	if (idx != -1) {
		minRelativeLV = name.substring(idx + 5); 	// get minLevel from the config 
		name = name.substring(0, idx); 				// remove "_minLevel_xxx" 
	} 	

	if (isToAddData){
		data = ", data = {" + dispatchEnemy + dispatchInterval + dispatchNum + maxNum + scale + isDestroyable + "}";
	}

    return "section = \"" + portal + "\", name = \"" + name + "\", minLevel = " + minLevel + ", maxLevel = " + maxLevel + ", minRelativeLV = " + minRelativeLV + ", maxRelativeLV = " + maxRelativeLV + ", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + data;

}

function addEnemyOnTrigger(portal, name, px, py, w, h) {
	// format config: enemy_<name of enemy>_minLevel_<enemy minLevel, default: 1>_maxLevel_<enemy maxLevel, default: 1> 
    var idx = name.indexOf("_");
	var minLevel = "nil"; 
	var maxLevel = "nil"; 
	var minRelativeLV = "nil"; 
	var maxRelativeLV = "nil";
	var state = "nil"; 
    
	name = name.substring(idx + 1); // remove "enemy" 
	
	idx = name.indexOf("_state"); // find if there is maxLevel config 
	if (idx != -1) {
		state = name.substring(idx + 7); // get maxLevel from the config 
		name = name.substring(0, idx); // remove "_maxLevel_xxx" 
	}

	idx = name.indexOf("_maxLevel"); // find if there is maxLevel config 
	if (idx != -1) {
		maxLevel = name.substring(idx + 10); // get maxLevel from the config 
		name = name.substring(0, idx); // remove "_maxLevel_xxx" 
	}
	
	idx = name.indexOf("_minLevel"); // find if there is minLevel config 
	if (idx != -1) {
		minLevel = name.substring(idx + 10); // get minLevel from the config 
		name = name.substring(0, idx); // remove "_minLevel_xxx" 
	}

	idx = name.indexOf("_maxRelativeLV"); 			// find if there is maxRelativeLV config 
	if (idx != -1) {
		maxRelativeLV = name.substring(idx + 15); 	// get maxLevel from the config 
		name = name.substring(0, idx); 				// remove "_maxLevel_xxx" 
	}

	idx = name.indexOf("_minRelativeLV"); 			// find if there is minRelativeLV config 
	if (idx != -1) {
		minRelativeLV = name.substring(idx + 15); 	// get minLevel from the config 
		name = name.substring(0, idx); 				// remove "_minLevel_xxx" 
	} 

    return "section = \"" + portal + "\", name = \"" + name + "\", minLevel = " + minLevel + ", maxLevel = " + maxLevel + ", minRelativeLV = " + minRelativeLV + ", maxRelativeLV = " + maxRelativeLV + ", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", state = " + state;
}

function addNPC(name, px, py, w, h) {
	// format config: "npc_<name of npc>_alias_<name of alias>_heading_<name of heading (default: right-front, right-front, right-back, left-front, left-back, right, or left)>_flip_<default: return, stay, freeze> 
	var idx = name.indexOf("_"); 
	var heading = "right-front"; 
	var flip = "return"; 
	var alias = "nil"; 
	var event = "nil"; 
	
	name = name.substring(idx + 1); // remove "npc"
	idx = name.indexOf("_event"); // find if there is event config 
	if (idx != -1) {
		event = "\"" +  name.substring(idx + 7) + "\""; // get event from the config 
		name = name.substring(0, idx); // remove "_event_xxx" 
	} 
	
	idx = name.indexOf("_flip"); // find if there is flip config 
	if (idx != -1) {
		flip = name.substring(idx + 6); // get flip from the config
		name = name.substring(0, idx); // remove "_flip_xxx" 
	} 
	
	idx = name.indexOf("_heading"); // find if there is heading config 
	if (idx != -1) {
		heading = name.substring(idx + 9); // get heading from the config 
		name = name.substring(0, idx); // remove "_heading_xxx" 
	} 
	
	idx = name.indexOf("_alias"); // find if there is alias config 
	if (idx != -1) {
		alias = name.substring(idx + 7); // get alias from the config 
		name = name.substring(0, idx); // remove "_alias_xxx" 
	} else {
		alias = name; // default alias is name 
	}
	
	var head1 = "right"; 
	var head2 = "front"; 
	idx = heading.indexOf("-"); 
	if (idx != -1) {
		head1 = heading.substring(0, idx); 
		head2 = heading.substring(idx + 1); 
	} else {
		head1 = heading; 
	}
	
    return "\nmap.npc[#map.npc + 1] = { name = \"" + name + "\", alias = \"" + alias + "\", eventTrigger = " + event + ", heading = {\"" + head1 + "\", \"" + head2 + "\"}, flip = \"" + flip + "\", x = " + px + ", y = " + py + ", width = " + w + ", height = " + h + ", section = " + section + ", " + dayPreference + appearOrder + "}";
}

function headerString() {
    preferences.rulerUnits = Units.PIXELS 
    preferences.typeUnits = TypeUnits.PIXELS 

    var docWidth = docRef.width.value; 
    var docHeight = docRef.height.value; 

    return "local map = {}\n\n" 
        + "map.background = {}\n"
        + "map.bgmask = {}\n" 
        + "map.portal = {}\n" 
        + "map.section = {}\n"
		+ "map.obstacle = {}\n" 
        + "map.building = {}\n"
        + "map.trap = {}\n" 
		+ "map.searchable = {}\n"
		+ "map.breakable = {}\n"		 
        + "map.object = {}\n" 
        + "map.enemyTrigger = {}\n"
        + "map.gateTrigger = {}\n"        
		+ "map.questTrigger = {}\n"
		+ "map.eventTrigger = {}\n" 
        + "map.enemy = {}\n" 
		+ "map.gate = {}\n" 
		+ "map.composition = {}\n"
        + "map.npc = {}\n\n"
        + "map.width = " + docWidth + "\n" 
        + "map.height = " + docHeight + "\n"; 
}

function footerString() {
    return "\n\nreturn map\n";
}

function getLayerInfo(layer) {
    layerName = layer.name; 

	// top left position of layer 
    tx = getObjectLeft(layer);
    ty = getObjectTop(layer); 

    res_width = getObjectWidth(layer);
    res_height = getObjectHeight(layer); 

	// center position of layer 
    pos_x = tx + res_width*0.5; 
    pos_y = ty + res_height*0.5; 
}

function getObjectLeft(element) {
    return element.bounds[0].as("px");
}

function getObjectTop(element) {
    return element.bounds[1].as("px");
}

function getObjectWidth(element) {
    var x = element.bounds[0].as("px");
    return element.bounds[2].as("px") - x; 
}

function getObjectHeight(element) {
    var y = element.bounds[1].as("px");
    return element.bounds[3].as("px") - y;
}

function saveFile() {
    var fileLineFeed = "Macintosh"
    if ($.os.indexOf("Windows") != -1) {
        fileLineFeed = "Windows" 
    } 

    var dir = docRef.path; 
    var fileName = docRef.name.split('.')[0]; 
    var fileOut = new File(dir + "/" + fileName + ".lua"); 
    fileOut.lineFeed = fileLineFeed 
    fileOut.open("w", "TEXT", "????") 
    fileOut.write(output) 
    fileOut.close() 
    alert(fileName + ".lua was saved at " + dir);
}
