var GAME_DESIGNER_MODE 	= 0;
var GAME_ARTIST_MODE 	= 1;
var MODE = 0;
var activeDocument;

var progressWindow;
var pBar;
var firstLevelLayersCount;

function showProgressBar(counter, count){
	if( progressWindow != null ){
		progressWindow.hide();
	}
	progressWindow = new Window('palette', "Updating..." + "(" + 
		counter + "/" + count + ")"); 
	pBar = progressWindow.add("progressbar", undefined, counter,
		count);
	pBar.preferredSize.width = 300;
	progressWindow.show();
}

function main() {
	// Init
	activeDocument = app.activeDocument;

	var box = new Window('dialog', "Your Mode:");  
  	var gaButton = box.add('button', undefined, "Game Artist");  
	var gdButton = box.add('button', undefined, "Game Designer"); 
	var cancelButton = box.add('button', undefined, "Cancel"); 

	// Get first level layers count
	firstLevelLayersCount = activeDocument.layers.length


	gaButton.onClick = function(){ 
		MODE = GAME_ARTIST_MODE;
		box.hide();
		showProgressBar(0, firstLevelLayersCount);
		traverse(activeDocument, true);
		alert("Finish update!");
	}

	gdButton.onClick = function(){ 
		MODE = GAME_DESIGNER_MODE;
		box.hide();
		showProgressBar(0, firstLevelLayersCount);
		traverse(activeDocument, true);
		alert("Finish update!");
	}

	box.show();
}

// Write the full or sub-full layer or group name
// Of the used asset specific for the Game Designer.
var GAME_DESIGNER_ASSET = [
	"start-position",
	"section-area",
	"trigger",
	"collision",
	"eventTrigger",
	"steep-sand",
];


var GAME_DESIGNER_ASSET_LENGTH = GAME_DESIGNER_ASSET.length;

function isGameDesignerAsset(layerOrGroupName){
	var i 		= 0;
	var result 	= false;

	for(i = 0; i < GAME_DESIGNER_ASSET_LENGTH ; i++){
		var index = layerOrGroupName.indexOf(GAME_DESIGNER_ASSET[i]);
		if(index != -1){
			result = true;
			break;
		}
	}

	return result;
}

function traverse(node, isFirstLevelLayers){
	var layers 		 = node.layers

	if(layers != null){
		var layersLength = layers.length

	    // Loop to get each layer on adobe file
	    for (var i = 0; i < layersLength; i++) {
	        var group 		= layers[i]; 
			var groupName 	= group.name; 
			var isVisible 	= group.visible;
			var isGDAsset 	= isGameDesignerAsset(groupName);

			if(isFirstLevelLayers){
				pBar.value = i + 1;
				showProgressBar(i + 1, firstLevelLayersCount);
			}
			// We wont traverse deeper for GDAsset
			if(isGDAsset){
				if(MODE == GAME_DESIGNER_MODE){
					group.visible = true;
				} else {
					group.visible = false;
				}
			} else {
				traverse(group, false);
			}
		}
	}
}

// Helper function to wait UI Update
function WaitForRedraw(){
	var eventWait = charIDToTypeID("Wait")
	var enumRedrawComplete = charIDToTypeID("RdCm")
	var typeState = charIDToTypeID("Stte")
	var keyState = charIDToTypeID("Stte")
	var desc = new ActionDescriptor()

	desc.putEnumerated(keyState, typeState, enumRedrawComplete)
	executeAction(eventWait, desc, DialogModes.NO)
}


main();

